@@ -6,6 +6,7 @@ import hildon
 import hildondesktop
 import os
 import cairo
+import pango
 import time
 import re
 import gettext
@@ -21,13 +22,17 @@ except IOError:
 		return arg
 	_ = retme
 
+ussd_languages = ["German", "English", "Italian", "French", "Spanish", "Dutch", "Swedish", "Danish", "Portuguese", "Finnish", "Norwegian", "Greek", "Turkish", "Reserved1", "Reserved2", "Unspecified"]
+
 # TODO Cutt off too long messages and show them in separate dialog
 # how TODO widget vertical minimum size policy
+# TODO interface for queryng from scripts. For example query from one widget can change color of another widget
 
 class USSD_Controller:
 	def __init__( self, widget ) :
 		self.widget = widget
-		self.default_config = ["", "", "", 0, "", 0, True, []]
+		# number, parser, chain, interval, regexp, width, execute_at_start, retry pattern, font, [name, always show], language 
+		self.default_config = ["", "", "", 0, "", 0, True, [], pango.FontDescription("Nokia Sans 18"), [_("Click to update"), False], 15]
 		self.config = self.default_config
 		self.timeout_version = 0
 		self.retry_version = 0
@@ -73,12 +78,21 @@ class USSD_Controller:
 			fconfig.write(str(i))
 			first = False
 		fconfig.write("\n")
+		fconfig.writelines(["#Font description\n", "font="+self.config[8].to_string(), "\n"])
+		fconfig.writelines(["#Font color\n", "text_color="+self.widget.get_text_color().to_string(), "\n"])
+		fconfig.writelines(["#Background color\n", "bg_color="+self.widget.get_bg_color().to_string(), "\n"])
+		fconfig.writelines(["#Widget name\n", "name="+self.config[9][0], "\n"])
+		fconfig.writelines(["#Always show widget name\n", "show_name="+str(self.config[9][1]), "\n"])
+		fconfig.writelines(["#USSD reply language\n", "language="+str(self.config[10]), "\n"])
 		fconfig.close()
 
 		fcntl.flock(lockf,fcntl.LOCK_UN)
 		lockf.close()
 
-	def get_config( self, id ):
+	def get_config(self):
+		return self.config
+
+	def read_config( self, id ):
 		try :
 			self.id = id
 			config = open(os.getenv("HOME")+"/.ussdWidget.conf","r")
@@ -98,7 +112,7 @@ class USSD_Controller:
 					# This is config for another instace
 					continue
 
-				line=line.split('=', 2)
+				line=line.split('=', 1)
 				
 				if len(line) != 2 :
 					error = True
@@ -114,6 +128,7 @@ class USSD_Controller:
 					try:
 						self.config[3] = int(line[1].strip())
 					except:
+						error = True
 						print _("Error reading config on line %(line)d. Integer expected.")%{"line":i}
 						continue
 				elif line[0] == "regexp" :
@@ -122,6 +137,7 @@ class USSD_Controller:
 					try:
 						self.config[5] = int(line[1].strip())
 					except:
+						error = True
 						print _("Error reading config on line %(line)d. Integer expected.")%{"line":i}
 						continue
 				elif line[0] == "query_at_start" :
@@ -138,12 +154,49 @@ class USSD_Controller:
 							try:
 								line[1][i] = int(line[1][i])
 							except:
+								error = True
 								print _("Error reading config on line %(line)d. Integer expected.")%{"line":i}
 							i += 1
 						self.config[7] = line[1]
 					else:
 						self.config[7] = []
-					continue
+						continue
+				elif line[0] == "font" :
+					try:
+						self.config[8] = pango.FontDescription(line[1].strip())
+					except:
+						error = True
+						print _("Error reading config on line %(line)d. Pango font description expected.")%{"line":i}
+						continue
+				elif line[0] == "bg_color" :
+					try:
+						self.widget.set_bg_color(gtk.gdk.color_parse(line[1].strip()))
+					except:
+						error = True
+						print _("Error reading config on line %(line)d. Expected color definition.")%{"line":i}
+				elif line[0] == "text_color" :
+					try:
+						self.widget.set_text_color(gtk.gdk.color_parse(line[1].strip()))
+					except:
+						error = True
+						print _("Error reading config on line %(line)d. Expected color definition.")%{"line":i}
+				elif line[0] == "name" :
+					self.config[9][0] = line[1].strip()
+				elif line[0] == "show_name" :
+					if line[1].strip() == "True" :
+						self.config[9][1] = True
+					else :
+						self.config[9][1] = False
+				elif line[0] == "language" :
+					try:
+						if int(line[1].strip()) >=0 and int(line[1].strip()) < len(ussd_languages):
+							self.config[10] = int(line[1].strip())
+						else:
+							error = True
+							print _("Error reading config on line %(line)d. Unknown language code.")%{"line":i}
+					except:
+						error = True
+						print _("Error reading config on line %(line)d. Integer expected.")%{"line":i}
 				else :
 					error = True
 					print _("Error reading config on line %(line)d. Unexpected variable: ")%{"line":i}+line[0]
@@ -164,7 +217,7 @@ class USSD_Controller:
 			return self.default_config
 
 	def on_show_settings( self, widget ) :
-		dialog = UssdConfigDialog(self.config)
+		dialog = UssdConfigDialog(self.config, self.widget.get_bg_color(), self.widget.get_text_color())
 
 		while True:
 			if dialog.run() != gtk.RESPONSE_OK :
@@ -209,16 +262,23 @@ class USSD_Controller:
 			dialog.regexp.get_text(),
 			dialog.widthEdit.get_value(),
 			dialog.query_at_start.get_active(),
-			retry
+			retry,
+			dialog.font,
+			[dialog.wname.get_text(), dialog.show_name.get_active()],
+			dialog.language.get_active()
 		]
 
+		widget.set_bg_color(dialog.bg_color)
+		widget.set_text_color(dialog.text_color)
+
 		self.save_config()
 
 		widget.set_width(self.config[5])	
 		self.reset_timed_renew()
-		
+		self.widget.label.modify_font(self.config[8])
+
 		dialog.destroy()
-    
+
 		# Before running this function widget wasn't configured
 		if self.config == self.default_config:
 			self.widget.set_text(_("Click to update"))
@@ -240,13 +300,17 @@ class USSD_Controller:
 			return False
 
 		print (_("serious problems in program logic"))
+		# This will force widget to show error message
+		self.cb_reply = ""
+		self.cb_ready = 1
+		self.process_reply()
 		return False
 
 
-	def call_external_script( self, ussd_code ):
+	def call_external_script( self, ussd_code, language ):
 		self.cb_ready = 0
 		self.cb_reply = "";
-		p = Popen(['/usr/bin/ussdquery.py', ussd_code], stdout=PIPE)
+		p = Popen(['/usr/bin/ussdquery.py', ussd_code, ussd_languages[language]], stdout=PIPE)
 		gobject.io_add_watch( p.stdout, gobject.IO_IN | gobject.IO_PRI | gobject.IO_HUP | gobject.IO_ERR , self.callback_ussd_data )
 
 	def ussd_renew(self, widget, event):
@@ -254,7 +318,7 @@ class USSD_Controller:
 			if self.config :
 				widget.processing = 1
 				widget.set_text(_("Processing"), 0)
-				self.call_external_script( self.config[0] )
+				self.call_external_script( self.config[0], self.config[10] )
 			else :
 				widget.processing = 0
 				widget.error = 1
@@ -322,10 +386,15 @@ class pHelpDialog(gtk.Dialog):
 		self.parent
 
 class UssdConfigDialog(gtk.Dialog):
-	def __init__(self, config):
+	def __init__(self, config, bg_color, text_color):
 		gtk.Dialog.__init__(self, _("USSD widget"), None, 
 			gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
 			(_("Save").encode("utf-8"), gtk.RESPONSE_OK))
+
+		self.font = config[8]
+		self.bg_color = bg_color
+		self.text_color = text_color
+
 		self.set_size_request(-1, 400)
 		self.ussdNumber = hildon.Entry(gtk.HILDON_SIZE_AUTO)
 		self.ussdNumber.set_text(config[0])
@@ -340,7 +409,23 @@ class UssdConfigDialog(gtk.Dialog):
 		self.widthEdit = hildon.NumberEditor(0, 1000)
 		self.widthEdit.set_value(config[5])
 		self.retryEdit = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-	
+
+
+		#selector = hildon.hildon_touch_selector_new_text()
+		selector = hildon.TouchSelector(text=True)
+		for i in ussd_languages:
+			selector.append_text(i)
+		self.language = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+		self.language.set_selector(selector)
+		self.language.set_active(config[10])
+		self.language.set_title(_("USSD reply language"))
+		self.language.set_size_request(-1, -1)
+
+		self.wname = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+		self.wname.set_text(config[9][0])
+		self.show_name = gtk.CheckButton(_("Always show name"))
+		self.show_name.set_active(config[9][1])
+
 		text = ""
 		for i in config[7]:
 			if text != "":
@@ -351,6 +436,14 @@ class UssdConfigDialog(gtk.Dialog):
 		self.query_at_start = gtk.CheckButton(_("Execute query on start"))
 		self.query_at_start.set_active(config[6])
 
+		self.fontButton = gtk.Button(_("Font"))
+		self.fontButton.connect("clicked", self.on_show_font_selection)
+
+		self.colorButton = gtk.Button(_("Background color"))
+		self.colorButton.connect("clicked", self.on_show_color_selection)
+		self.textColorButton = gtk.Button(_("Text color"))
+		self.textColorButton.connect("clicked", self.on_show_text_color_selection)
+		
 		phelp = gtk.Button("?")
 		phelp.connect("clicked", self.on_show_phelp)
 
@@ -377,8 +470,17 @@ class UssdConfigDialog(gtk.Dialog):
 		numberBox.add(self.ussdNumber)
 		vbox.add(numberBox)
 
+		vbox.add(self.language)
+
 		vbox.add(self.query_at_start)
 
+		nameBox = gtk.HBox()
+		self.show_name.set_size_request(100, -1)
+		self.wname.set_size_request(100, -1)
+		nameBox.add(self.show_name)
+		nameBox.add(self.wname)
+		vbox.add(nameBox)
+
 		parserBox = gtk.HBox()
 		parserLabel = gtk.Label(_("Parser"))
 		parserLabel.set_alignment(0,0)
@@ -442,6 +544,12 @@ class UssdConfigDialog(gtk.Dialog):
 		retryBox.add(retryhelp)
 		vbox.add(retryBox)
 		vbox.add(self.retryEdit)		
+		
+		viewBox = gtk.HBox()
+		viewBox.add(self.fontButton)
+		viewBox.add(self.textColorButton)
+		viewBox.add(self.colorButton)
+		vbox.add(viewBox)
 
 		self.show_all()
 		self.parent
@@ -482,6 +590,31 @@ class UssdConfigDialog(gtk.Dialog):
 		dialog.run()
 		dialog.destroy()
 
+	def on_show_color_selection (self, event):
+		colorDialog = gtk.ColorSelectionDialog(_("Choose background color"))
+		colorDialog.colorsel.set_current_color(self.bg_color)
+		if colorDialog.run() == gtk.RESPONSE_OK :
+			self.bg_color = colorDialog.colorsel.get_current_color()
+		colorDialog.destroy()
+
+	def on_show_text_color_selection (self, event):
+		colorDialog = gtk.ColorSelectionDialog(_("Choose text color"))
+		colorDialog.colorsel.set_current_color(self.text_color)
+		if colorDialog.run() == gtk.RESPONSE_OK :
+			self.text_color = colorDialog.colorsel.get_current_color()
+		colorDialog.destroy()
+	
+	def on_show_font_selection (self, event):
+		fontDialog = gtk.FontSelectionDialog(_("Choose a font"))
+		fontDialog.set_font_name(self.font.to_string())
+
+		if fontDialog.run() != gtk.RESPONSE_OK :
+			fontDialog.destroy()
+			return
+
+		self.font = pango.FontDescription (fontDialog.get_font_name())
+        	fontDialog.destroy()
+
 def smart_split_string (str, query) :
 	word = ""
 	result = []
@@ -541,6 +674,8 @@ class UssdWidgetPlugin(hildondesktop.HomePluginItem):
 		hildondesktop.HomePluginItem.__init__(self)
 	
 		self.processing = 0
+		self.bg_color=gtk.gdk.color_parse('#000000')
+		self.text_color=gtk.gdk.color_parse('#ffffff')
 		self.error = 0
 		self.timeout_version = 0
 
@@ -558,7 +693,6 @@ class UssdWidgetPlugin(hildondesktop.HomePluginItem):
 		self.set_settings(True)
 		self.connect("show-settings", self.controller.on_show_settings)
 		self.label = gtk.Label("")
-		self.set_text(_("Click to update"))		
 		
 		self.vbox.add(self.label)
 		self.vbox.set_child_packing(self.label, False, False, 0, gtk.PACK_START)
@@ -570,9 +704,9 @@ class UssdWidgetPlugin(hildondesktop.HomePluginItem):
 		self.vbox.show_all()
 
 	def do_show(self):
-		config = self.controller.get_config(self.get_applet_id())
+		config = self.controller.read_config(self.get_applet_id())
 		self.set_width(config[5])
-
+		self.set_text("")		
 		if config[6]:
 			self.controller.ussd_renew(self, None)
 
@@ -595,6 +729,10 @@ class UssdWidgetPlugin(hildondesktop.HomePluginItem):
 		else :
 			if showfor == -1 :
 				self.text = text
+		
+		config = self.controller.get_config()
+		if config[9][1] or text == "":
+			text = config[9][0]+text
 		self.label.set_text(text)
 
 	def get_text(self):
@@ -606,6 +744,19 @@ class UssdWidgetPlugin(hildondesktop.HomePluginItem):
 		else :
 			self.label.set_width_chars(-1)
 
+	def set_bg_color(self, color):
+		self.bg_color = color
+
+	def get_bg_color(self):
+		return self.bg_color
+
+	def set_text_color(self, color):
+		self.label.modify_fg(gtk.STATE_NORMAL, color)		
+		self.text_color = color
+
+	def get_text_color(self):
+		return self.text_color
+
 	def _expose(self, event):
 		cr = self.window.cairo_create()
 
@@ -637,7 +788,7 @@ class UssdWidgetPlugin(hildondesktop.HomePluginItem):
 		if self.processing :
 			bg_color=fg_color
 		else :
-			bg_color=gtk.gdk.color_parse('#000000')
+			bg_color=self.bg_color
 
 		cr.set_source_rgba (bg_color.red / 65535.0, bg_color.green/65535.0, bg_color.blue/65535.0, 0.7)
 		cr.fill_preserve ()
